/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.search;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.part.ViewPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.config.IFindTaxaAndNamesConfigurator;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.cdm.persistence.query.MatchMode;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.ContextListenerAdapter;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.navigation.search.SearchBar.SearchOption;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>SearchResultView class.</p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 19.01.2009
 * @version 1.0
 */
public class SearchResultView extends ViewPart implements IConversationEnabled{

	private static Object[] EMPTY = new Object[0];

	private class ContextListener extends ContextListenerAdapter{
		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IContextListener#contextStop(org.eclipse.ui.IMemento, org.eclipse.core.runtime.IProgressMonitor)
		 */
		@Override
		public void contextStop(IMemento memento, IProgressMonitor monitor) {
			monitor.subTask("Getting rid of search results");
			AbstractUtility.hideView(SearchResultView.this);
		}
	}

	/** Constant <code>ID="eu.etaxonomy.taxeditor.navigation.searc"{trunked}</code> */
	public static final String ID =
			"eu.etaxonomy.taxeditor.navigation.search.searchResultView"; //$NON-NLS-1$

	private TableViewer resultViewer;

	private ConversationHolder conversation;

	private Text searchString;

	private Text configurationLabel;

	private Text status;

	private SearchJob searchJob;

	private IContextListener contextListener;

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {

		conversation = CdmStore.createConversation();
		contextListener = new ContextListener();
		CdmStore.getContextManager().addContextListener(contextListener);

		GridLayout layout = new GridLayout();
		layout.marginWidth = 0;
		layout.marginHeight = 0;

		parent.setLayout(layout);

		Composite infoComposite = createInfoComposite(parent);
		infoComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));

		resultViewer = new TableViewer(parent, SWT.NONE);
		resultViewer.setContentProvider(new ArrayContentProvider());
		resultViewer.setLabelProvider(new SearchResultLabelProvider());
		resultViewer.addDoubleClickListener(new IDoubleClickListener() {
			@Override
            public void doubleClick(DoubleClickEvent event) {
				NavigationUtil.executeEditHandler();
			}
		});

		resultViewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		getSite().setSelectionProvider(resultViewer);

		// register context menu
		MenuManager menuMgr = new MenuManager();
		menuMgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		getSite().registerContextMenu(menuMgr, resultViewer);

		Control control = resultViewer.getControl();
		Menu menu = menuMgr.createContextMenu(control);
		control.setMenu(menu);
	}

	private Composite createInfoComposite(Composite parent){
		Composite composite = new Composite(parent, SWT.NULL);

		composite.setLayout(new GridLayout(2, false));

		Label searchStringLabel = new Label(composite, SWT.NULL);
		searchStringLabel.setText("Search String:");

		searchString = new Text(composite, SWT.NULL);
		searchString.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		searchString.setEditable(false);
//		searchString.setText("                                               ");

		Label configurationDescriptionLabel = new Label(composite, SWT.NULL);
		configurationDescriptionLabel.setText("Search for:");

		configurationLabel = new Text(composite, SWT.WRAP);
		configurationLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		configurationLabel.setEditable(false);

		Label statusLabel = new Label(composite, SWT.NULL);
		statusLabel.setText("Status:");

		status = new Text(composite, SWT.NULL);
		status.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		status.setEditable(false);

		return composite;
	}

	/**
	 * <p>performSearch</p>
	 *
	 * @param configurator a {@link eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator} object.
	 */
	public void performSearch(IFindTaxaAndNamesConfigurator configurator){
		setPartName("Search: '" + configurator.getTitleSearchString() + "'");

		searchString.setText(configurator.getTitleSearchString());

		List<String> includedEntities = new ArrayList<String>();
		if(configurator.isDoTaxa()) {
            includedEntities.add(SearchOption.TAXON.getLabel());
        }
		if(configurator.isDoSynonyms()) {
            includedEntities.add(SearchOption.SYNONYM.getLabel());
        }
		if(configurator.isDoNamesWithoutTaxa()) {
            includedEntities.add(SearchOption.NAME.getLabel());
        }
		if(configurator.isDoTaxaByCommonNames()){
			includedEntities.add(SearchOption.COMMON_NAME.getLabel());
		}
		configurator.setMatchMode(MatchMode.LIKE);

		String includedEntitiesString = "";
		for (int i = 0; i < includedEntities.size(); i++){
			includedEntitiesString += includedEntities.get(i);
			if(i < includedEntities.size() -1){
				includedEntitiesString += ", ";
			}
		}

		configurationLabel.setText(includedEntitiesString);

		status.setText("Searching...");

		searchJob = new SearchJob(Display.getCurrent(), configurator);
		searchJob.schedule();

	}

	/**
	 * <p>displaySearchResult</p>
	 *
	 * @param result a {@link java.util.List} object.
	 */
	protected void displaySearchResult(List<UuidAndTitleCache<IdentifiableEntity>> result) {
		if(result.size() > 0){
			resultViewer.setInput(result);
			status.setText(result.size() + " entities found");
		}else{
			resultViewer.setInput(EMPTY);
			status.setText("Search returned no results");
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	/** {@inheritDoc} */
	@Override
	public void setFocus() {
		//logger.warn("Setting focus to search result viewer");
		conversation.bind();
		// pass focus to resultViewer
		resultViewer.getControl().setFocus();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	/**
	 * <p>getConversationHolder</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	@Override
    public ConversationHolder getConversationHolder() {
		return this.conversation;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	/** {@inheritDoc} */
	@Override
    public void update(CdmDataChangeMap changeEvents) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#dispose()
	 */
	/** {@inheritDoc} */
	@Override
	public void dispose() {
		super.dispose();
		conversation.close();
		if(searchJob != null) {
            searchJob.cancel();
        }
		CdmStore.getContextManager().removeContextListener(contextListener);
	}

	/**
	 *
	 * @author n.hoffmann
	 * @created Feb 2, 2010
	 * @version 1.0
	 */
	class SearchJob extends Job{

		private final IFindTaxaAndNamesConfigurator configurator;

		private final Display display;

		/**
		 * @param name
		 */
		public SearchJob(Display display, IFindTaxaAndNamesConfigurator configurator) {
			super("Performing Search");
			this.display = display;
			this.configurator = configurator;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.IProgressMonitor)
		 */
		@Override
		protected IStatus run(IProgressMonitor monitor) {
			monitor.beginTask("", 100);
			monitor.worked(20);

			final List<UuidAndTitleCache<IdentifiableEntity>> searchResult = CdmStore.getSearchManager().findTaxaAndNames(configurator);
			monitor.worked(40);

			if(! monitor.isCanceled()){
				display.asyncExec(new Runnable() {
					@Override
                    public void run() {
						displaySearchResult(searchResult);
					}
				});
			}else{
				display.asyncExec(new Runnable() {
					@Override
                    public void run() {
						status.setText("Cancelled");
					}
				});
			}
			monitor.done();
			return Status.OK_STATUS;
		}

	}
}
